<?php
require_once '../datos/Conexion.clase.php';
date_default_timezone_set("America/Lima");

class Monitoreo extends Conexion {
    public function AgregarMonitoreo(
        $fecha_registro, $hora_registro, $id_pozo, $id_propietario, $lectura_caudalimetro, 
        $nivel_estatico, $nivel_dinamico, $conductividad_agua, $imagen_inicial, 
        $latitud, $longitud, $estado, $codigo_usuario, $id_realm_monitoreo, $nombre_pozo, $nombre_comercial
    ) {
        try {
            // Verificar si la conexión a la base de datos está establecida
            if (!$this->dblink) {
                throw new Exception("No se pudo establecer la conexión a la base de datos.");
            }
    
            // Iniciar la transacción
            $this->dblink->beginTransaction();
    
            // Obtener el correlativo para el nuevo monitoreo
            $sql = "SELECT numero + 1 as num FROM correlativo WHERE tabla = 'monitoreo_pozo';";
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetch();
    
            if ($resultado && $sentencia->rowCount() > 0) {
                $id_monitoreo = $resultado['num'];
    
                // Obtener la fecha y hora actuales
                $fecha_subida = date('Y-m-d');
                $hora_subida = date("H:i:s");
    
                // Preparar el SQL para insertar los datos de monitoreo
                $sql = "INSERT INTO monitoreo_pozo (
                    id_monitoreo, fecha_registro, hora_registro, 
                    fecha_registro_subida, hora_registro_subida, 
                    id_pozo, id_propietario, lectura_caudalimetro, nivel_estatico, 
                    nivel_dinamico, conductividad_agua, imagen_inicial, 
                    latitud, longitud, estado, codigo_usuario, id_realm_monitoreo, nombre_pozo, nombre_comercial
                ) VALUES (
                    :p_id_monitoreo, :p_fecha_registro, :p_hora_registro, 
                    :p_fecha_registro_subida, :p_hora_registro_subida, 
                    :p_id_pozo, :p_id_propietario, :p_lectura_caudalimetro, :p_nivel_estatico, 
                    :p_nivel_dinamico, :p_conductividad_agua, :p_imagen_inicial, 
                    :p_latitud, :p_longitud, :p_estado, :p_codigo_usuario, :p_id_realm_monitoreo, :p_nombre_pozo, :p_nombre_comercial
                )";
    
                // Preparar la sentencia
                $sentencia = $this->dblink->prepare($sql);
    
                // Bindear los parámetros
                $sentencia->bindParam(":p_id_monitoreo", $id_monitoreo);
                $sentencia->bindParam(":p_fecha_registro", $fecha_registro);
                $sentencia->bindParam(":p_hora_registro", $hora_registro);
                $sentencia->bindParam(":p_fecha_registro_subida", $fecha_subida);
                $sentencia->bindParam(":p_hora_registro_subida", $hora_subida);
                $sentencia->bindParam(":p_id_pozo", $id_pozo);
                $sentencia->bindParam(":p_id_propietario", $id_propietario);
                $sentencia->bindParam(":p_lectura_caudalimetro", $lectura_caudalimetro);
                $sentencia->bindParam(":p_nivel_estatico", $nivel_estatico);
                $sentencia->bindParam(":p_nivel_dinamico", $nivel_dinamico);
                $sentencia->bindParam(":p_conductividad_agua", $conductividad_agua);
                $sentencia->bindParam(":p_imagen_inicial", $imagen_inicial, PDO::PARAM_LOB); // BLOB para imagen
                $sentencia->bindParam(":p_latitud", $latitud);
                $sentencia->bindParam(":p_longitud", $longitud);
                $sentencia->bindParam(":p_estado", $estado);
                $sentencia->bindParam(":p_codigo_usuario", $codigo_usuario);
                $sentencia->bindParam(":p_id_realm_monitoreo", $id_realm_monitoreo);
                $sentencia->bindParam(":p_nombre_pozo", $nombre_pozo);
                $sentencia->bindParam(":p_nombre_comercial", $nombre_comercial);

    
                // Ejecutar la sentencia
                $sentencia->execute();
    
                // Actualizar el correlativo después de insertar el monitoreo
                $sql = "UPDATE correlativo SET numero = :p_numero WHERE tabla = 'monitoreo_pozo';";
                $sentencia = $this->dblink->prepare($sql);
                $sentencia->bindParam(":p_numero", $id_monitoreo);
                $sentencia->execute();
    
                // Confirmar la transacción
                $this->dblink->commit();
            } else {
                // No se encontró el correlativo, lanzar excepción
                throw new Exception("No se pudo obtener el correlativo para el monitoreo.");
            }
    
            return true;
        } catch (Exception $e) {
            // Realizar rollback si ocurre un error
            if ($this->dblink->inTransaction()) {
                $this->dblink->rollBack();
            }
            // Registrar el error en el log o retornar un mensaje descriptivo
            error_log("Error en AgregarMonitoreo: " . $e->getMessage());
            throw $e;
        }
    }

    public function listarMonitoreos($fecha_inicio, $fecha_fin, $id_propietario) {
        try {
            
            $sql = "SELECT * 
                    FROM monitoreo_pozo 
                    WHERE fecha_registro BETWEEN :p_fecha_inicio AND :p_fecha_fin
                    AND id_propietario = :p_id_propietario;";
    
            // Asegúrate de que la conexión esté abierta
            if (!$this->dblink) {
                throw new Exception("No se pudo establecer la conexión a la base de datos.");
            }
    
            $sentencia = $this->dblink->prepare($sql);
            
            // Asignación de los valores a los parámetros
            $sentencia->bindParam(':p_fecha_inicio', $fecha_inicio);
            $sentencia->bindParam(':p_fecha_fin', $fecha_fin);
            $sentencia->bindParam(':p_id_propietario', $id_propietario, PDO::PARAM_INT);
            
            $sentencia->execute();
            $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
            
            return $resultado;
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function listarMonitoreosCompletos($fecha_inicio, $fecha_fin) {
        try {
            
            $sql = "SELECT * 
                    FROM monitoreo_pozo 
                    WHERE fecha_registro BETWEEN :p_fecha_inicio AND :p_fecha_fin;";
    
            // Asegúrate de que la conexión esté abierta
            if (!$this->dblink) {
                throw new Exception("No se pudo establecer la conexión a la base de datos.");
            }
    
            $sentencia = $this->dblink->prepare($sql);
            
            // Asignación de los valores a los parámetros
            $sentencia->bindParam(':p_fecha_inicio', $fecha_inicio);
            $sentencia->bindParam(':p_fecha_fin', $fecha_fin);
            
            $sentencia->execute();
            $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
            
            return $resultado;
        } catch (Exception $e) {
            throw $e;
        }
    }
    

}