<?php
require_once '../datos/Conexion.clase.php';
date_default_timezone_set("America/Lima");

class Pozo extends Conexion {

    public function __construct() {
        parent::__construct();  // Asegúrate de que el constructor de la clase padre (Conexion) se ejecute
    }

    public function listarPozos() {
        try {
            $sql = "select
                        p.id_pozo,
                        p.nombre_pozo,
                        p.cantidad_inicial,
                        p.latitud_ubicacion,
                        p.longitud_ubicacion,
                        p.codigo_pozo_sis_aqua,
                        p.id_propietario,
						p2.nombre_comercial,
                        p.imagen_inicial,
                        p.estado_app as estado,
                        p.codigo_usuario_movil as codigo_usuario,
                        p.estado_pozo as estadoPozo,
                        p.fecha_registro_app as fecha_registro,
                        p.hora_registro_app as hora_registro
                        
                    from 
                        Pozo_propietario p
                    join
                    	Propietario p2 
                    on
                    	 p.id_propietario = p2.id_propietario
                    where
                        p.estado = 1";

            // Asegúrate de que la conexión esté abierta
            if (!$this->dblink) {
                throw new Exception("No se pudo establecer la conexión a la base de datos.");
            }

            $sentencia = $this->dblink->prepare($sql);
            $sentencia->execute();
            $resultado = $sentencia->fetchAll(PDO::FETCH_ASSOC);
            return $resultado;
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function ActualizarPozo($cantidad_inicial, $latitud, $longitud, $imagen_inicial, $estado_app, $codigo_usuario_movil , $estado_pozo, $fecha_registro_app, $hora_registro_app, $id_pozo) {
        try {
            $hora = date("H:i:s");
            $fecha = date('Y-m-d');
            // Iniciar la transacción
            $this->dblink->beginTransaction();
    
            $sql = "UPDATE Pozo_propietario 
                    SET 
                    cantidad_inicial = :p_cantidad_inicial,
                    latitud_ubicacion = :p_latitud,
                    longitud_ubicacion = :p_longitud,
                    imagen_inicial = :p_imagen_inicial,
                    estado_app = :p_estado_app,
                    fecha_registro = :p_fecha,
                    hora_registro = :p_hora,
                    codigo_usuario_movil = :p_codigo_usuario_movil,
                    estado_pozo = :p_estado_pozo,
                    fecha_registro_app = :p_fecha_registro_app,
                    hora_registro_app = :p_hora_registro_app
                    WHERE 
                    id_pozo = :p_id_pozo";
    
            // Asegúrate de que la conexión esté abierta
            if (!$this->dblink) {
                throw new Exception("No se pudo establecer la conexión a la base de datos.");
            }
    
            $sentencia = $this->dblink->prepare($sql);
            $sentencia->bindParam(":p_cantidad_inicial", $cantidad_inicial);
            $sentencia->bindParam(":p_latitud", $latitud);
            $sentencia->bindParam(":p_longitud", $longitud);
            $sentencia->bindParam(":p_imagen_inicial", $imagen_inicial);
            $sentencia->bindParam(":p_estado_app", $estado_app);
            $sentencia->bindParam(":p_fecha", $fecha); 
            $sentencia->bindParam(":p_hora", $hora);   
            $sentencia->bindParam(":p_codigo_usuario_movil", $codigo_usuario_movil);   
            $sentencia->bindParam(":p_estado_pozo", $estado_pozo);   
            $sentencia->bindParam(":p_fecha_registro_app", $fecha_registro_app);   
            $sentencia->bindParam(":p_hora_registro_app", $hora_registro_app);   
            $sentencia->bindParam(":p_id_pozo", $id_pozo);
            $sentencia->execute();
    
            // Confirma la transacción
            $this->dblink->commit();
    
            return true;
        } catch (Exception $e) {
            // Realiza un rollback en caso de error
            if ($this->dblink->inTransaction()) {
                $this->dblink->rollBack();
            }
            throw $e;
        }
    }
    
}
